//
//  SidebarBluetoothStatusCardView.swift
//  USB Meter
//

import SwiftUI

struct SidebarBluetoothStatusCardView: View {
    let tint: Color
    let statusText: String

    var body: some View {
        VStack(alignment: .leading, spacing: 6) {
            HStack {
                Label("Bluetooth", systemImage: "bolt.horizontal.circle.fill")
                    .font(.footnote.weight(.semibold))
                    .foregroundColor(tint)
                Spacer()
                Text(statusText)
                    .font(.caption.weight(.semibold))
                    .foregroundColor(.secondary)
            }
            Text("Refer to this adapter state while walking through the Bluetooth and Device troubleshooting steps.")
                .font(.caption2)
                .foregroundColor(.secondary)
        }
        .padding(14)
        .meterCard(tint: tint, fillOpacity: 0.22, strokeOpacity: 0.26, cornerRadius: 18)
    }
}