//
//  MeterLiveMetricRange.swift
//  USB Meter
//

import Foundation

struct MeterLiveMetricRange {
    let minLabel: String
    let maxLabel: String
    let minValue: String
    let maxValue: String
}